import subprocess
from settings import logger
from bzrrecipe import BzrRecipe
import os


class DputRunner:
    DEBFULLNAME = None
    DEBEMAIL = None
    versionFormat = ('{PACKAGE_VERSION}+bzr{BZR_REVISION}+' +
                     'pkg{PACKAGING_REVISION}~{DISTRIBUTION}1')
    distributions = []
    jenkinsBuildNumber = ''
    ppas = []

    def __init__(self, ppas, trunk, packagingBranch,
                 DEBFULLNAME, DEBEMAIL, revision=None,
                 versionFormat=None,
                 distributions=None, jenkinsBuildNumber=''):
        self.DEBFULLNAME = DEBFULLNAME
        self.DEBEMAIL = DEBEMAIL
        if versionFormat:
            self.versionFormat = versionFormat
        self.distributions = distributions
        self.jenkinsBuildNumber = jenkinsBuildNumber
        self.ppas = ppas
        self.trunk = trunk
        self.packagingBranch = packagingBranch
        self.revision = revision

    def formatVersionString(self, **kwargs):
        try:
            return self.versionFormat.format(**kwargs)
        except KeyError as error:
            logger.error('Unkown variable used in format string: ' +
                         error.message)
            return None

    def doDput(self, recipe, version_string, changelog_entry, distribution,
               ppa):
        logger.info('Going to dput version: ' + version_string)
        #export DEBFULLNAME=
        os.putenv('DEBFULLNAME', self.DEBFULLNAME)
        #export DEBEMAIL=
        os.putenv('DEBEMAIL', self.DEBEMAIL)
        native_hack = ['/usr/share/pbuilderjenkins/hooks/H40native_hack.py']
        logger.debug('Calling: %r', native_hack)
        subprocess.check_call(native_hack, cwd=recipe.getTrunkDir())
        #dch -v $version $changelog -D $distribution
        dch_call = ['dch', '-v', version_string, changelog_entry,
                    '-D', distribution]
        logger.debug('Calling: %r', dch_call)
        subprocess.check_call(dch_call, cwd=recipe.getTrunkDir())
        name = recipe.getSourceName() + '_' + version_string
        #debuild -S -d
        debuild_call = ['debuild', '--no-tgz-check', '-S', '-d']
        logger.debug('Calling: %r', debuild_call)
        subprocess.check_call(debuild_call, cwd=recipe.getTrunkDir())
        #dput $ppa *${version_string}_source.changes
        dput_call = ['dput', ppa, name + '_source.changes']
        logger.debug('Calling: %r', dput_call)
        subprocess.check_call(dput_call, cwd=recipe.getBuildDir())
        #rm -f _source.ppa.upload
        rm_call = ['rm', '-f', name + '_source.ppa.upload']
        logger.debug('Calling: %r', rm_call)
        subprocess.check_call(rm_call, cwd=recipe.getBuildDir())
        #revert the previous dch calls to get a clean changelog
        logger.debug('Reverting the change made by dch call')
        recipe.revert()

    def dput(self):
        if not self.distributions:
            logger.error('No distribution specified. Aborting dput.')
            return False

        recipe = BzrRecipe(self.trunk, self.packagingBranch, self.revision)

        for distribution in self.distributions:
            for ppa in self.ppas:
                formatedVersionString = self.formatVersionString(
                         PACKAGE_VERSION=recipe.getPackageVersion(),
                         BZR_REVISION=recipe.getTrunkRevno(),
                         PACKAGING_REVISION=recipe.getPackagingBranchRevno(),
                         JENKINS_BUILD_NUMBER=self.jenkinsBuildNumber,
                         DISTRIBUTION=distribution)
                if not formatedVersionString:
                    logger.error(
                            'Aborting dput as no format string was specified')
                    return False
                try:
                    self.doDput(recipe, formatedVersionString,
                                "Automatic build of revision " +
                                str(recipe.getTrunkRevno()),
                                distribution,
                                ppa)
                except subprocess.CalledProcessError as e:
                    logger.error('Command %s returned non-zero exit status %d',
                                 e.cmd, e.returncode)
                    return False
        recipe.cleanup()
        return True
