#!/usr/bin/env python
from argparse import ArgumentParser
import argparse
from settings import logger
from bzrrecipe import BzrRecipe
from dputrunner import DputRunner
import os
import sys


class StringSplitAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        setattr(namespace, self.dest, values.split(','))


if __name__ == "__main__":
    parser = ArgumentParser(
        description="Generate a branch based package version.")
    parser.add_argument('-a', '--packaging-branch', required=False,
                        help="Packaging branch to be used for dput-ing")
    parser.add_argument('-t', '--target-branch', required=True,
                        help="Target (main) branch to be used for dput-ing")
    parser.add_argument('-v', '--revision', default=None,
                        help="Revision of source branch that should be merged")
    parser.add_argument('-r', '--trunk-rev', default=None,
                        help="Override trunk revision number for package " +
                        "version string")
    parser.add_argument('-p', '--packaging-rev', default=None,
                        help="Override packaging revision number for pacakge " +
                        "version string")
    parser.add_argument('-d', '--distribution', required=True,
                        help="Distribution (e.g. quantal)")
    parser.add_argument('-b', '--build-number', required=True,
                        help="Build number")
    parser.add_argument('-m', '--merge', action='store_true',
                        help="Build is a merge, automatically increment " +
                        "trunk rev (cannot be combined with --trunk-rev)")
    args = vars(parser.parse_args())
    if (args['merge'] == True) and (not args['trunk_rev'] is None):
        print("ERROR: -m/--merge and -r/--trunk-rev cannot be combined")
        parser.print_help()
        sys.exit(-1)

    dputrunner = DputRunner(ppas=None,
                            trunk=args['target_branch'],
                            packagingBranch=args['packaging_branch'],
                            DEBFULLNAME=None,
                            DEBEMAIL=None,
                            revision=args['revision'],
                            versionFormat='{PACKAGE_VERSION}bzr{BZR_REVISION}'
                            + 'pkg{PACKAGING_REVISION}{DISTRIBUTION}{BUILD}',
                            distributions=None)
    recipe = BzrRecipe(dputrunner.trunk, dputrunner.packagingBranch,
                       dputrunner.revision)

    # recipe.getTrunkRevno() will return the latest revno in launchpad.
    # For the package version string, we want the next version so as to
    # sync up with the MP candidate after it is merged to trunk.
    trunk_rev = recipe.getTrunkRevno()
    if args['merge'] == True:
        trunk_rev += 1
    if not args['trunk_rev'] is None:
        trunk_rev = args['trunk_rev']

    packaging_rev = recipe.getPackagingBranchRevno()
    if not args['packaging_rev'] is None:
        packaging_rev = args['packaging_rev']

    version = dputrunner.formatVersionString(
                         PACKAGE_VERSION=recipe.getPackageVersion(),
                         BZR_REVISION=trunk_rev,
                         PACKAGING_REVISION=packaging_rev,
                         DISTRIBUTION=args['distribution'],
                         BUILD=args['build_number'])
    recipe.cleanup()
    print("version: %s" % (version))
